#!/usr/bin/env python
#coding:utf-8
# Author        : tuxpy
# Email         : q8886888@qq.com.com
# Last modified : 2016-07-13 16:36:40
# Filename      : history.py
# Description   : 
from __future__ import print_function, unicode_literals
import multiprocessing
import os
import json
import time

LOCK = multiprocessing.RLock()

HISTORY_FILE = os.path.join(os.path.expanduser('~'), '.vacuum_history')

def get():
    if not os.path.exists(HISTORY_FILE):
        return {}

    with open(HISTORY_FILE, 'rb') as fd:
        return json.loads(fd.read())

def save(history):
    with LOCK:
        with open(HISTORY_FILE, 'wb') as fd:
            fd.write(json.dumps(history))

def update_latest_exception_time(application_name, warning_rule_name, timestamp):
    old_history = get()
    key = '%s:%s:%s' % (application_name, warning_rule_name, 'latest_exception_time')
    old_history[key] = timestamp
    save(old_history)

def get_latest_exception_time(application_name, warning_rule_name):
    key = '%s:%s:%s' % (application_name, warning_rule_name, 'latest_exception_time')
    return get().get(key)

def update_latest_send_sms_time():
    old_history = get()
    old_history['latest_send_sms_time'] = time.time()
    save(old_history)

def get_latest_send_sms_time():
    return get().get('latest_send_sms_time')




#coding:utf-8
from __future__ import print_function
import os
import sys
import threading
import time
import datetime
import traceback
import inspect
import json
from vacuum import notifiy
from vacuum import api

LOCK = threading.RLock()

APP_NAME = None


def set_app_name(name):
    global APP_NAME
    APP_NAME = name

def get_app_name():
    global APP_NAME
    return APP_NAME

def installThreadExcepthook():
    init_old = threading.Thread.__init__

    def init(self, *args, **kwargs):
        init_old(self, *args, **kwargs)
        run_old = self.run

        def run_with_except_hook(*args, **kw):
            try:
                run_old(*args, **kw)
            except (KeyboardInterrupt, SystemExit):
                raise
            except:
                sys.excepthook(*sys.exc_info())
        self.run = run_with_except_hook
    threading.Thread.__init__ = init

def write_log(data):
    global APP_NAME
    app_name = APP_NAME or os.path.basename(sys.argv[0])
    log_file = get_log_file_name(app_name)

    if os.path.exists(log_file) and os.stat(log_file).st_size > 50 * 1024 * 1024: # 如果日志文件大于50M, 则结束当前程序
        print('logfile to big, please check exception', file = sys.stderr)
        notifiy.send_email('%s日志文件过大' % (get_app_name(), ), '请检查异常', api.get_email_receivers(get_app_name()))
        notifiy.send_sms(api.get_sms_receivers(get_app_name()), 'big exception')
        os._exit(1)

    with LOCK:
        print(data['exception_info'], file = sys.stderr)
        with open(log_file, 'a') as fd:
            fd.write(json.dumps(data) + '\n')

def error(ex, tb_info, func_args = None, **ext_param):
    func_args = func_args or {}
    data = {
            'func_name': '<manual>',
            'func_args': func_args,
            'exception_name': ex.__class__.__name__,
            'exception_info': tb_info,
            'timestamp': time.time(),
            'ext_param': ext_param
            }

    write_log(data)

def log_exception(excType, excValue, tb, ):
    exc = traceback.format_exception(excType, excValue, tb)
    exc = ''.join(exc)

    frame = get_last_tb(tb).tb_frame
    file_name = frame.f_code.co_filename
    func_name = frame.f_code.co_name
    func_args = {}
    for key in inspect.getargvalues(frame).args:
        func_args[key] = str(inspect.getargvalues(frame).locals[key])

    data = {
        'func_name': func_name,  # 调用函数名
        'func_args': func_args,  # 函数调用
        'exception_name': excValue.__class__.__name__,  # 异常名称
        'exception_info': exc,  # 异常信息
        # 'ext_param': '', #自定义异常内容参数
        'timestamp': time.time(),  # 异常时间
    }

    write_log(data)


def get_last_tb(tb):
    while tb.tb_next != None:
        tb = tb.tb_next
    return tb

def get_log_file_name(app_name):
    # 日志路径格式: $DSB_WARNING_LOG/{application_name}/YYYY/MM/DD_exception.log
    log_path = os.environ.get('DSB_WARNING_LOG')
    if log_path:
        if log_path.endswith('/'):
            log_path = log_path[:-1]
    else:
        log_path = '/var/log/vacuum'

    file_name = '{}/{}/{:%Y/%m/%d}_exception.log'.format(
        log_path, app_name, datetime.datetime.now())

    path = file_name[:file_name.rfind('/')]
    if not os.path.exists(path):
        os.makedirs(path)

    return file_name


sys.excepthook = [log_exception, sys.__excepthook__][0]
installThreadExcepthook()

if __name__ == "__main__":
    # export DSB_WARNING_LOG=abc
    # export DSB_WARNING_LOG=
    env = 'DSB_WARNING_LOG'
    log_path = os.environ.get(env)
    if log_path:
        print(env, log_path)
    # quit()

    def foo(a, b, c):
        a = 1 / 0

    threading.Thread(target=foo, args=(1, 2, 9)).start()
    # foo()



from mail import Mail
class ADDRESSBOOKMail:
    '''
    Init appid,appkey,sign_type(Optional)
    '''
    def __init__(self,configs):
        self.configs = configs
        self.appid = configs['appid']
        self.appkey = configs['appkey']
        self.sign_type = ''
        if configs['sign_type'] != '':
            self.sign_type = configs['sign_type']
        self.address = ''
        self.target = ''
    
    '''
    set name and address
    '''
    def set_address(self, address, name=''):
        self.address = name+'<'+address+'>'

    '''
    set target
    '''
    def set_address_book(self, target):
        self.target = target

    '''
    build request array
    '''
    def build_request(self):
        request = {}
        '''
        set subscribe address
        '''
        request['address'] = self.address

        '''
        set target addressbook
        '''
        if self.target != '':
            request['target'] = self.target
        return request

    '''
    @subscribe
    '''
    def subscribe(self):
        mail_configs = {}
        '''
        set appid and appkey
        '''
        mail_configs['appid'] = self.appid
        mail_configs['appkey'] = self.appkey

        '''
        set sign_type,if is set
        '''
        if self.sign_type != '':
            mail_configs['sign_type'] = self.sign_type

        '''
        init mail class
        '''
        addressbook = Mail(mail_configs)

        '''
        build request and send email and return the result
        '''
        return addressbook.subscribe(self.build_request())

    '''
    @unsubscribe
    '''
    def unsubscribe(self):
        mail_configs = {}
        '''
        set appid and appkey
        '''
        mail_configs['appid'] = self.appid
        mail_configs['appkey'] = self.appkey

        '''
        set sign_type,if is set
        '''
        if self.sign_type != '':
            mail_configs['sign_type'] = self.sign_type

        '''
        init mail class
        '''
        addressbook = Mail(mail_configs)

        '''
        build request and send email and return the result
        '''
        return addressbook.unsubscribe(self.build_request())




'''
Submail addressbook/mail/unsubscribe API demo
SUBMAIL SDK Version 1.0.1 --python
copyright 2011 - 2014 SUBMAIL
'''
from app_configs import MAIL_CONFIGS
from address_book_mail import ADDRESSBOOKMail
'''
init MESSAGEXsend class
'''
addressbook = ADDRESSBOOKMail(MAIL_CONFIGS)

'''
Required para
The First para: recipient email address
The second para: recipient name(optional)
'''
addressbook.set_address('leo@apple.cn','leo')

'''
Optional para
set target addressbook sign : Optional
default value: subscribe
'''
#addressbook.set_address_book('unsubscribe')
print addressbook.unsubscribe()



#!/usr/bin/env python
#coding:utf-8
# Author        : tuxpy
# Email         : q8886888@qq.com.com
# Last modified : 2016-07-19 14:36:53
# Filename      : api.py
# Description   : 
from __future__ import print_function, unicode_literals

from .config import get_config
import requests
import sys

CONFIG = get_config()

def api_request(method, url, *args, **kwargs):
    url = 'http://' + (CONFIG.get('api_host') or raw_input('please enter api host: ').strip()) + url
    headers = {
            'User-Agent': 'vacuum client',
            'Api-Token': CONFIG.get('api_token') or raw_input('please enter api token: ').strip(),
            }
    response = requests.request(method, url, headers = headers, *args, **kwargs)
    json_data = response.json()
    if json_data['code'] != 0:
        sys.stderr.write(json_data['msg'] + '\n')
        sys.exit(1)

    return response.json()['data']

def fetch_config():
    return api_request('GET', '/api/config')['config']

def _get_receviers(application_name):
    return api_request('GET', '/api/receivers', params = {'application_name': application_name})['receivers']

def get_email_receivers(application_name):
    return _get_receviers(application_name)['email']

def get_sms_receivers(application_name):
    return _get_receviers(application_name)['sms']




'''
Mail configs start
mail_configs['appid']
mail_configs['appkey']
mail_configs['sign_type']
'''
MAIL_CONFIGS = {}
'''
Mail appid
'''
MAIL_CONFIGS['appid'] = 'your_mail_app_id'

'''
Mail appkey
'''
MAIL_CONFIGS['appkey'] = 'your_mail_app_key'

'''
Mail Sign type
md5/sha1/normal
'''
MAIL_CONFIGS['sign_type'] = 'md5'

'''
Mail configs end
'''

'''
Message configs start
message_configs['appid']
message_configs['appkey']
message_configs['sign_type']
'''
MESSAGE_CONFIGS = {}
'''
Message appid
'''
MESSAGE_CONFIGS['appid'] = '10997'

'''
Message appkey
'''
MESSAGE_CONFIGS['appkey'] = '379bc3fdf9c44c81ee5f0a8a372863ea'

'''
Message Sign type
md5/sha1/normal
'''
MESSAGE_CONFIGS['sign_type'] = 'md5'

'''
Message configs end
'''



#!/usr/bin/env python
#coding:utf-8
# Author        : tuxpy
# Email         : q8886888@qq.com.com
# Last modified : 2016-07-12 15:43:06
# Filename      : config.py
# Description   : 
from __future__ import print_function, unicode_literals

from vacuum import datatype
import os
import sys
import json

RC_FILE = os.path.join(os.path.expanduser('~'), '.vacuumrc')

def _get_config_from_rc():
    if not os.path.exists(RC_FILE):
        return {}

    with open(RC_FILE, 'rb') as fd:
        content = fd.read()
        return json.loads(content)

def _get_log_path():
    return os.environ.get('DSB_WARNING_LOG', '/var/log/vacuum')

def save_config(config):
    with open(RC_FILE, 'wb') as fd:
        fd.write(json.dumps(config, indent = 4))

def get_config():
    config_from_rc = _get_config_from_rc()
    if not config_from_rc:
        return {}

    config = datatype.ObjectDict(config_from_rc)
    config['log_path'] = _get_log_path()

    return config




'''
Submail mail/xsend API demo
SUBMAIL SDK Version 1.0.1 --python
copyright 2011 - 2014 SUBMAIL
'''
from mail_xsend import MAILXsend
from app_configs import MAIL_CONFIGS

'''
init MESSAGEXsend class
'''
submail = MAILXsend(MAIL_CONFIGS)

'''
Optional para
The First para: recipient email address
The second para: recipient name(optional)
@Multi-para
'''
submail.add_to('leo@submail.cn','leo')

'''
Optional para
set addressbook sign : Optional
add addressbook contacts to Multi-Recipients
@Multi-para
'''
#submail.add_address_book('subscribe')

'''
Optional para
set sender address and name
The First para: sender email address
The second para: sender display name (optional)
'''
#submail.set_sender('no-reply@submail.cn','SUBMAIL')

'''
Optional para
set reply address
'''
#submail.set_reply('service@submail.cn')

'''
Optional para
set email subject
'''
#submail.set_subject('test SDK')

'''
Required para
set project sign
'''
submail.set_project('uigGk1')

'''
Optional para
submail email text content filter
@Multi-para
'''
submail.add_var('name','leo')
submail.add_var('age','32')

'''
Optional para
submail email link content filter
@Multi-para
'''
submail.add_link('developer','http://submail.cn/chs/developer')
submail.add_link('store','http://submail.cn/chs/store')

'''
Optional para
email headers
@Multi-para
'''
submail.add_header('X-Accept','zh-cn')
submail.add_header('X-Mailer','leo App')
print submail.xsend()



#!/usr/bin/env python
#coding:utf-8
# Author        : tuxpy
# Email         : q8886888@qq.com.com
# Last modified : 2016-07-12 16:47:33
# Filename      : log.py
# Description   : 

from __future__ import print_function, unicode_literals
from cattle import Cattle
from vacuum import api
from vacuum import history 
from vacuum import notifiy
from vacuum.config import get_config
import datetime
import hashlib
import json
import os
import re
import time
import zipfile

CONFIG = get_config()

def compile_warning_rules(warning_rules):
    for _rule in warning_rules:
        name = _rule['name']
        if name[0] != '^':
            name = '^.*' + name

        if name[-1] != '$':
            name = name + '.*$'

        _rule['name_re'] = re.compile(r'%s' % (name, ))

    return warning_rules

compile_warning_rules(CONFIG['warning_rules'])

class Dust(object):
    def __init__(self, record):
        self.record = record.strip()
        self.data = json.loads(record)
        for name, value in self.data.items():
            setattr(self, name, value)

        self.warning_rule = self.get_warning_rule()
        self.data['warning_rule_name'] = self.warning_rule_name = self.warning_rule['name']

    def out(self):
        return json.dumps(self.data)

    def __repr__(self):
        return "< Dust warning_rule: {warning_rule_name}, exception_name: {exception_name}, timestamp: {timestamp}>".format(
                 **self.data)

    def get_id(self, *params):
        """通过日志的参数来生成该条日志的标记, 标记不唯一。标记相同的日志表示是同一条日志。如果不传，则采用exception_name, func_name来确定标记"""
        params = sorted(params or ['exception_name', 'func_name'])
        md5_instance = hashlib.md5()
        for param in params:
            md5_instance.update(param + str(self.data[param]))

        return md5_instance.hexdigest()

    def get_warning_rule(self):
        exception_name = self.exception_name.lower().strip()
        default_rule = None
        
        for _rule in CONFIG.warning_rules:
            if _rule['name'] == 'default': 
                default_rule = _rule

            if _rule['name_re'].match(exception_name):
                return _rule

        assert default_rule, 'miss default warning rule'

        return default_rule

    def ok(self, condition):
        """第一版本只判断and关系"""
        if not condition:
            return True

        condition = self.unwind_condition(condition)
        for name, value, opera in condition:
            opera_method = getattr(self, 'is_' + opera)
            if not opera_method(name, value):
                return False

        return True

    def unwind_condition(self, condition):
        result = []
        for name, value_expression in condition.items():
            if isinstance(value_expression, dict) and all([key[0] == '$' for key in value_expression.keys()]):
                for opera, value in value_expression.items():
                    opera = opera[1:]
                    assert hasattr(self, 'is_' + opera), 'opera not defined'

                    result.append((name, value, opera))

            else:
                value = value_expression
                result.append((name, value, 'eq'))

        return result

    def __getitem__(self, name):
        return getattr(self, name)

    def is_eq(self, name, value):
        return self[name] == value

    def is_neq(self, name, value):
        return self[name] != value

    def is_gte(self, name, value):
        return self[name] >= value

    def is_gt(self, name, value):
        return self[name] > value

    def is_lt(self, name, value):
        return self[name] < value

    def is_lte(self, name, value):
        return self[name] < value

class Logger(object):
    def __init__(self, _file):
        assert os.path.isfile(_file), 'not found logger file: %s' % (_file, )
        self._file = _file

    def _iter_line(self):
        for line in self.fd:
            yield line

    def init_iter(self):
        if hasattr(self, 'fd'):
            self.fd.close()

        self.fd = open(self._file, 'rb')
        self._iter = self._iter_line()

    def find(self, condition = None):
        self.init_iter()

        meet_dusts = []
        for dust in self:
            if dust.ok(condition):
                meet_dusts.append(dust)

        return meet_dusts


    def __iter__(self):
        return self

    def __exit__(self, *args, **kwargs):
        self.fd.close()

    def next(self):
        return Dust(self._iter.next())

    def size(self):
        return os.stat(self._file).st_size

class DustGroup(object):
    def __init__(self, dusts):
        self.dusts = dusts

    def ls(self):
        return self.dusts

    def out(self):
        return '\n'.join([ dust.out() for dust in self.dusts])

    def by(self, *params):
        """通过指定params来对dust做分组, 返回
        [
        [dust, ...],
        [dust, ...],
        ]
        """
        dust_map = {}
        for dust in self.dusts:
            dust_map.setdefault(dust.get_id(*params), []).append(dust)

        return dust_map.values()

def ls(what_date = None):
    """
    列出所有的日志, 返回[(application1 name, exception log), ]
    """
    logs = []
    what_date = what_date or time.strftime("%Y/%m/%d")
    for application in os.listdir(CONFIG.log_path):
        exception_log = os.path.join(CONFIG.log_path, application, what_date + "_exception.log")
        if not os.path.exists(exception_log):
            continue
        logs.append((application, exception_log))

    return logs

def report_analysis_summay(application_name, dusts):
    warning_rule = dusts[0].warning_rule
    subject = '{application_name} 出异常了，规则名: {rule_name}, 每{intervals}秒只能出现{threshold}次异常, 实际出现了{actual_dust_numbers}'.format(
        rule_name = warning_rule['name'],
        intervals = warning_rule['intervals'],
        threshold = warning_rule['threshold'],
        application_name = application_name,
        actual_dust_numbers = len(dusts),
        )
    print(subject)
    content = DustGroup(dusts).out()
    print(content)

    notifiy.send_email(subject, content, api.get_email_receivers(application_name))
    notifiy.send_sms(api.get_sms_receivers(application_name), 'exception')

def analysis_same_dust(application_name, dusts):
    if not dusts:
        return
    warning_rule = dusts[0].warning_rule
    exception_begin_timestamp = dusts[0].timestamp
    need_warning_dusts = []
    max_warning_dusts = []

    latest_exception_time = history.get_latest_exception_time(application_name, warning_rule['name'])
    
    for dust in dusts:
        if latest_exception_time and dust.timestamp < latest_exception_time: # 如果当前异常发生在上一次找到的有异常的异常日志前，则表示之前已经预警过了，不再计算
            continue

        if dust.timestamp >= exception_begin_timestamp + warning_rule['intervals']: # 如果超过了某一个时间间隔，则需要重新记数
            exception_begin_timestamp = dust.timestamp
            over_threshold = len(need_warning_dusts) > warning_rule['threshold']
            over_threshold and history.update_latest_exception_time(application_name, warning_rule['name'], dust.timestamp)

            if over_threshold and len(need_warning_dusts) >  len(max_warning_dusts): # 把异常数据最多的那个时间段记录下来
                max_warning_dusts = need_warning_dusts

            need_warning_dusts = []

        need_warning_dusts.append(dust)

    need_warning_dusts = max_warning_dusts or need_warning_dusts
    if len(need_warning_dusts) > warning_rule['threshold']:
        history.update_latest_exception_time(application_name, warning_rule['name'], dusts[-1].timestamp)

    else:
        return

    return need_warning_dusts

def choice_best_timestamp_range(application_name):
    """根据历史日志扫描结果来选择一个最佳的时间范围.找出上次扫描中异常时间最早的时间作为该次扫描的日志起启时间"""
    all_latest_exception_time  = [ history.get_latest_exception_time(application_name, _rule['name']) for _rule in CONFIG.warning_rules]
    all_latest_exception_time = [ _time for _time in all_latest_exception_time if _time]
    if not all_latest_exception_time:
        return time.time() - 30 * 60 # 默认时间是前30分钟

    min_latest_exception_time = min([ _time for _time in all_latest_exception_time if _time])
    return max(min_latest_exception_time, time.time() - 30 * 60)

def analysis(application_name, log_file):
    logger = Logger(log_file)
    dusts = logger.find({
        'timestamp': {"$gte": choice_best_timestamp_range(application_name)}, # 取出最近30分钟内的数据
        })

    dust_group = DustGroup(dusts)
    dusts_groups = dust_group.by('warning_rule_name')
    for dusts in dusts_groups:
        exception_dusts = analysis_same_dust(application_name, dusts)
        if not exception_dusts:
            continue

        report_analysis_summay(application_name, exception_dusts)

### qiniu ######
def get_bucket():
    return Cattle(CONFIG.qiniu['ak'], CONFIG.qiniu['sk']).get_bucket(CONFIG.qiniu['bucket'])

def upload(file_path, cdn_path):
    bucket = get_bucket()

    log_zip_path = file_path + '.zip'
    log_zip = zipfile.ZipFile(log_zip_path, 'w', compression = zipfile.ZIP_DEFLATED)
    log_zip.write(file_path, arcname = file_path[len(CONFIG.log_path) + 1:])
    log_zip.close()

    bucket.put_file(log_zip_path, key = os.path.basename(log_zip_path), prefix = cdn_path)

def sync2cdn():
    yesterday = (datetime.date.today() - datetime.timedelta(1)).strftime("%Y/%m/%d")
    for application_name, log_file in ls(yesterday):
        cdn_path = os.path.dirname(log_file)[len(CONFIG.log_path) + 1: ] + '/'
        upload(log_file, cdn_path)




#!/usr/bin/env python
#coding:utf-8
# Author        : tuxpy
# Email         : q8886888@qq.com.com
# Last modified : 2016-07-12 16:31:03
# Filename      : parse.py
# Description   : 
from __future__ import unicode_literals, unicode_literals




'''
Submail message/xsend API demo
SUBMAIL SDK Version 1.0.1 --python
copyright 2011 - 2014 SUBMAIL
'''
from message_xsend import MESSAGEXsend
from app_configs import MESSAGE_CONFIGS

'''
init MESSAGEXsend class
'''
submail = MESSAGEXsend(MESSAGE_CONFIGS)

'''
Optional para
recipient cell phone number
@Multi-para
'''
submail.add_to('18616761881')

'''
Optional para
set addressbook sign : Optional
add addressbook contacts to Multi-Recipients
@Multi-para
'''
#submail.add_address_book('subscribe')

'''
Required para
set message project sign
'''
submail.set_project('kZ9Ky3')

'''
Optional para
submail email text content filter
@Multi-para
'''
submail.add_var('code','198276')
print submail.xsend()



#!/usr/bin/env python
#coding:utf-8
# Author        : tuxpy
# Email         : q8886888@qq.com.com
# Last modified : 2016-07-12 15:47:12
# Filename      : datetype.py
# Description   : 
from __future__ import print_function, unicode_literals


class ObjectDict(dict):
    def __getattr__(self, name):
        return self[name]

    def __setattr__(self, name, value):
        self[name] = value




#!/usr/bin/env python
#coding:utf-8
# Author        : tuxpy
# Email         : q8886888@qq.com.com
# Last modified : 2016-07-14 10:10:18
# Filename      : notifiy.py
# Description   : 
from __future__ import print_function, unicode_literals

from vacuum.config import get_config
from vacuum import history
from third_party.submail.mail_send import MAILSend
from third_party import ucpaas
import time


SUBMAIL_CONFIG = get_config()['submail']
UCPAAS_CONFIG = get_config()['ucpaas']

def get_submail_config():
    return get_config()['submail']

def get_ucpaas_config():
    return get_config()['ucpaas']

def _utf8(s):
    if isinstance(s, unicode):
        return s.encode('utf-8')

    else:
        return s

def send_email(subject, content, to):
    mailer = MAILSend(SUBMAIL_CONFIG)
    [ mailer.add_to(_to) for _to in to]
    mailer.set_sender(SUBMAIL_CONFIG['sender'])
    mailer.set_subject(_utf8(subject))
    mailer.set_text(_utf8(content))
    print(mailer.send())

def send_sms(to, *params):
    latest_send_sms_time = history.get_latest_send_sms_time()
    if latest_send_sms_time and time.time() - latest_send_sms_time < SUBMAIL_CONFIG['send_interval']:
        print('最近刚发过短信')
        return

    for phone in to:
        ucpaas.send_sms(UCPAAS_CONFIG['appid'], UCPAAS_CONFIG['sid'],
                UCPAAS_CONFIG['token'], phone, UCPAAS_CONFIG['template_id'], *params)

    history.update_latest_send_sms_time()




#!/usr/bin/env python
#coding:utf-8
# Author        : tuxpy
# Email         : q8886888@qq.com.com
# Last modified : 2016-07-14 14:55:30
# Filename      : setup.py
# Description   : 

from setuptools import setup

setup(
        name = 'vacuum',
        version = '0.3.1',
        author = 'duoshoubang',
        packages = [
            'vacuum',
            'vacuum.third_party',
            'vacuum.third_party.submail',
            ],
        install_requires = ['cattle_qiniu', 'requests'],
        description = 'JJ Bob',
        scripts = ['bin/vacuum'],
        )




#!/usr/bin/env python
#coding:utf-8
# Author        : tuxpy
# Email         : q8886888@qq.com.com
# Last modified : 2016-01-29 15:27:35
# Filename      : ucpaas.py
# Description   : 
from __future__ import print_function

import requests
import base64
import time
import json
import hashlib

__ALL__ = ['send_sms']

def made_authorization(sid, t):
    return base64.encodestring(sid + ':' + t).strip()

def made_sig(sid, token, t):
    return hashlib.md5(sid + token + t).hexdigest().upper()


def _send_sms(appid, sid, token, phone, template_id, *params):
    t = time.strftime('%Y%m%d%H%M%S')
    params_str = ','.join([ str(param) for param in params])
    headers = {
            'Accept'            :       'application/json',
            'Content-Type'      :       'application/json;charset=utf-8',
            'Authorization'     :       made_authorization(sid, t),
            }

    json_data =  '{"templateSMS":{ "appId":"%s","to":"%s","templateId":"%s","param":"%s"}}' % (appid,  phone, template_id, params_str) 

    api_url =  'https://api.ucpaas.com/{app_version}/Accounts/{sid}/Messages/templateSMS?sig={sig}'.format(
            app_version = '2014-06-30', sid = sid, sig = made_sig(sid, token, t))
    response = requests.request(**{
        'method'            :       'POST',
        'data'              :       json_data,
        'url'               :       api_url,
        'headers'           :       headers,
        })

    success = response.json()['resp']['respCode'] == '000000'

    if not success:
        print(response.content)

    return success


def send_sms(*args, **kwargs):
    try:
        return _send_sms(*args, **kwargs)
    except Exception as ex:
        return False

